// 
// Author: 	Sergey Bogdanov <sergey.bogdanov@gmail.com>
// Url:		http://www.sergeybogdanov.com
//
using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace CEKiosk
{
	/// <summary>
	/// Summary description for Kiosk.
	/// </summary>
	public class Kiosk
	{
		Form _mainForm;

		IntPtr _hwnd = IntPtr.Zero;
		private IntPtr Hwnd
		{
			get
			{
				if (_hwnd == IntPtr.Zero)
				{
					_mainForm.Capture = true;
					_hwnd = GetCapture();
					_mainForm.Capture = false;
				}
				
				return _hwnd;
			}
		}

		public Kiosk(Form mainForm)
		{
			if (mainForm == null) throw new System.ArgumentNullException();
			_mainForm = mainForm;
		}

		public void Start()
		{
			_mainForm.ControlBox = false;

			EnableToolbar(false);

            _mainForm.WindowState = FormWindowState.Maximized;

			SetWindowPos(Hwnd, 0, 0, 0, Screen.PrimaryScreen.Bounds.Width,
				                        Screen.PrimaryScreen.Bounds.Height, SWP_SHOWWINDOW);

			Button sip = new Button();
			sip.Width = 40;
			sip.Height = 20;
			sip.Text = "SIP";
			sip.Click += new EventHandler(sip_Click);
			sip.Location = new Point(_mainForm.ClientRectangle.Width - sip.Width - 2, _mainForm.ClientRectangle.Height - sip.Height - 2);
			_mainForm.Controls.Add(sip);
            _mainForm.Resize += new EventHandler(_mainForm_Resize); //Visual Studio 2005 resizes after load. We are more stupborn.
		}

        void _mainForm_Resize(object sender, EventArgs e)
        {
            SetWindowPos(Hwnd, 0, 0, 0, Screen.PrimaryScreen.Bounds.Width,
                                        Screen.PrimaryScreen.Bounds.Height, SWP_SHOWWINDOW);
        }

		public void Stop()
		{
            EnableToolbar(true);			
		}


		public static void EnableToolbar(bool enabled)
		{
			IntPtr p = FindWindow("HHTaskBar", null);
			if (p == IntPtr.Zero)
			{
				return;
			}
            
			IntPtr i = ShowWindow(p, enabled ? SW_SHOW : SW_HIDE);
			bool b = EnableWindow(p, enabled);
		}

		public void ShowSIP(bool value)
		{
			SipShowIM(value ? SIPF_ON : SIPF_OFF);
		}
		
		private void sip_Click(object sender, EventArgs e)
		{
			SIPINFO s = new SIPINFO();
			SipGetInfo(s);
			ShowSIP(!((s.fdwFlags & 1) == SIPF_ON));
		}

		#region P/Invoke declarations

		const uint SIPF_OFF = 0x0;
		const uint SIPF_ON = 0x1;

		const int HWND_TOPMOST		= -1;
		const int SWP_SHOWWINDOW	= 0x0040;

		const int SW_HIDE			= 0;
		const int SW_SHOW			= 1;

		[DllImport("coredll.dll")]
		private static extern bool SetWindowPos(IntPtr hwnd, int hwnd2, int x, int y, int cx, int cy, int uFlags);

		[DllImport("coredll.dll")]
		private static extern IntPtr FindWindow(
			string lpClassName,
			string lpWindowName);

		[DllImport("coredll.dll")]
		private static extern IntPtr GetCapture();

		[DllImport("coredll.dll")]
		private static extern IntPtr ShowWindow(IntPtr hWnd, int visible);

		[DllImport("coredll.dll")]
		private static extern bool EnableWindow(IntPtr hwnd, bool enabled);

		[DllImport("coredll.dll")]
		private extern static void SipShowIM(uint dwFlag);


		private class SIPINFO
		{
			public SIPINFO()
			{
				cbSize = Marshal.SizeOf(typeof(SIPINFO));
			}


			public int cbSize;
			public int fdwFlags;
			public RECT rcVisibleDesktop;
			public RECT rcSipRect;
			public int dwImDataSize;
			public int pvImData;
		}

		private struct RECT 
		{
			public int left;
			public int top;
			public int right;
			public int bottom;
		}

		[DllImport("coredll.dll")]
		private extern static bool SipGetInfo(SIPINFO sipInfo);
		#endregion
	}
}
